import typer
from pathlib import Path
from configparser import ConfigParser
import os

config = ConfigParser()


APP_NAME = "bing-img-search"
app_dir = typer.get_app_dir(APP_NAME)
app_dir = Path(app_dir)
app_dir.mkdir(parents=True, exist_ok=True)
config_path = app_dir / "config.cfg"


def write_config_file():
    config_file = open(config_path, 'w')
    config.write(config_file)
    config_file.close()


# Initialize Default Config
def initialize_config():
    if not os.path.exists(config_path):
        config['main'] = {'MAX_RESULTS': '250', 'GROUP_SIZE': '50'}
        write_config_file()


def write_config(section, key, value):
    config.read(config_path)
    config.set(section, key, value)
    write_config_file()


def read_config(section, key, return_type):
    config.read(config_path)
    if return_type == 'int':
        return config.getint(section, key)
    elif return_type == 'str':
        return config.get(section, key)




from requests import exceptions
import requests
from PIL import Image
import os
from pathlib import Path
import os
from urllib.parse import urlparse


from bingoset.utilities.checks import check_config

# MAX_RESULTS = 250
# GROUP_SIZE = 50


def get_images_bing(search_term, MAX_RESULTS, GROUP_SIZE):
    API_KEY = check_config()

    URL = "https://api.cognitive.microsoft.com/bing/v7.0/images/search"

    EXCEPTIONS = {
        IOError,
        FileNotFoundError,
        exceptions.RequestException,
        exceptions.HTTPError,
        exceptions.ConnectionError,
        exceptions.Timeout,
    }

    headers = {"Ocp-Apim-Subscription-Key": API_KEY}
    params = {"q": search_term, "offset": 0, "count": GROUP_SIZE}

    # make the search
    print("[INFO] searching Bing API for '{}'".format(search_term))
    search = requests.get(URL, headers=headers, params=params)
    search.raise_for_status()

    results = search.json()
    estNumResults = min(results["totalEstimatedMatches"], MAX_RESULTS)
    print("[INFO] {} total results for '{}'".format(estNumResults, search_term))

    # initialize the total number of images downloaded thus far
    total = 0

    # loop over the estimated number of results in `GROUP_SIZE` groups
    for offset in range(0, estNumResults, GROUP_SIZE):
        # update the search parameters using the current offset, then
        # make the request to fetch the results
        print(
            "[INFO] making request for group {}-{} of {}...".format(
                offset, offset + GROUP_SIZE, estNumResults
            )
        )
        params["offset"] = offset
        search = requests.get(URL, headers=headers, params=params)
        search.raise_for_status()
        results = search.json()
        print(
            "[INFO] saving images for group {}-{} of {}...".format(
                offset, offset + GROUP_SIZE, estNumResults
            )
        )

        # loop over the results
        for v in results["value"]:
            # try to download the image
            try:
                # make a request to download the image
                print("[INFO] fetching: {}".format(v["contentUrl"]))
                r = requests.get(v["contentUrl"], timeout=30)

                # build the path to the output image
                # ext = v["contentUrl"][v["contentUrl"].rfind(".") :]
                url_path = urlparse(v["contentUrl"]).path
                ext = os.path.splitext(url_path)[1]

                dataset_path = Path.cwd() / "dataset" / search_term
                path_out_img = dataset_path / f"{str(total).zfill(8)}{ext}"
                dataset_path.mkdir(parents=True, exist_ok=True)

                # write the image to disk
                f = open(path_out_img, "wb")
                f.write(r.content)
                f.close()

            # catch any errors that would not unable us to download the
            # image
            except Exception as e:
                # check to see if our exception is in our list of
                # exceptions to check for
                print(e)
                if type(e) in EXCEPTIONS:
                    print("[INFO] skipping: {}".format(v["contentUrl"]))
                    continue

            try:
                im = Image.open(path_out_img)
            except IOError:
                # filename not an image file, so it should be ignored
                print("[INFO] deleting: {}".format(path_out_img))
                os.remove(path_out_img)
                continue

            # update the counter
            total += 1





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bingoset', 'bingoset.utilities']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=7.1.2,<8.0.0', 'requests>=2.23.0,<3.0.0', 'typer[all]>=0.1.1,<0.2.0']

entry_points = \
{'console_scripts': ['bingoset = bingoset.main:app']}

setup_kwargs = {
    'name': 'bingoset',
    'version': '0.1.1',
    'description': '',
    'long_description': '<h4 align="center">\n    <a href="https://github.com/Akshay090/bingoset">\n        <img src="https://raw.githubusercontent.com/Akshay090/bingoset/master/.github/bingoset-banner.png" alt="bingoset" />\n    </a>\n    <br>\n    <br>\n     CLI Toolkit to quickly create image dataset using Bing Image Search API\n\n![Twitter Follow](https://img.shields.io/twitter/follow/aks2899?style=social)\n</h4>\n\n# Welcome to BingoSet \n\n\n## Install\n\n```sh\npip3 install bingoset\n```\n## Set-up\n\nGet your Bingo Image Search API key and execute below command\n\n```sh\nbingoset set-api-key YOUR_BING_API_KEY_HERE\n```\n\n## Usage \n\n```sh\nbingoset q pikachu\n```\nThis will download 250 (default) images of pikachu into a directory called dataset (default)\n\n## Additional Config\n\n```sh\nbingoset set-max-results 100\n```\nChange the number of images you want to download, eg : 100\n\n```sh\nbingoset set-group-size 20\n```\nChange the group size of images you want to download, eg : 100\n',
    'author': 'Akshay Ashok',
    'author_email': 'aks28id@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Akshay090/bingoset',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
